@@ -3,9 +3,12 @@ import re
 import sys
 import textwrap
 from typing import Any
+from typing import cast
+from typing import Dict
 from typing import Iterator
 from typing import List
 from typing import Optional
+from typing import Sequence
 from typing import Tuple
 from typing import Union
 
@@ -19,6 +22,7 @@ from _pytest import python
 from _pytest.outcomes import fail
 from _pytest.pytester import Testdir
 from _pytest.python import _idval
+from _pytest.python import idmaker
 
 
 class TestMetafunc:
@@ -35,10 +39,11 @@ class TestMetafunc:
         @attr.s
         class DefinitionMock(python.FunctionDefinition):
             obj = attr.ib()
+            _nodeid = attr.ib()
 
         names = fixtures.getfuncargnames(func)
         fixtureinfo = FuncFixtureInfoMock(names)  # type: Any
-        definition = DefinitionMock._create(func)  # type: Any
+        definition = DefinitionMock._create(func, "mock::nodeid")  # type: Any
         return python.Metafunc(definition, fixtureinfo, config)
 
     def test_no_funcargs(self) -> None:
@@ -72,7 +77,7 @@ class TestMetafunc:
         pytest.raises(ValueError, lambda: metafunc.parametrize("y", [5, 6]))
 
         with pytest.raises(TypeError, match="^ids must be a callable or an iterable$"):
-            metafunc.parametrize("y", [5, 6], ids=42)  # type: ignore[arg-type] # noqa: F821
+            metafunc.parametrize("y", [5, 6], ids=42)  # type: ignore[arg-type]
 
     def test_parametrize_error_iterator(self) -> None:
         def func(x):
@@ -90,7 +95,7 @@ class TestMetafunc:
         metafunc = self.Metafunc(func)
         # When the input is an iterator, only len(args) are taken,
         # so the bad Exc isn't reached.
-        metafunc.parametrize("x", [1, 2], ids=gen())  # type: ignore[arg-type] # noqa: F821
+        metafunc.parametrize("x", [1, 2], ids=gen())  # type: ignore[arg-type]
         assert [(x.funcargs, x.id) for x in metafunc._calls] == [
             ({"x": 1}, "0"),
             ({"x": 2}, "2"),
@@ -102,7 +107,7 @@ class TestMetafunc:
                 r" Exc\(from_gen\) \(type: <class .*Exc'>\) at index 2"
             ),
         ):
-            metafunc.parametrize("x", [1, 2, 3], ids=gen())  # type: ignore[arg-type] # noqa: F821
+            metafunc.parametrize("x", [1, 2, 3], ids=gen())  # type: ignore[arg-type]
 
     def test_parametrize_bad_scope(self) -> None:
         def func(x):
@@ -113,7 +118,7 @@ class TestMetafunc:
             fail.Exception,
             match=r"parametrize\(\) call in func got an unexpected scope value 'doggy'",
         ):
-            metafunc.parametrize("x", [1], scope="doggy")
+            metafunc.parametrize("x", [1], scope="doggy")  # type: ignore[arg-type]
 
     def test_parametrize_request_name(self, testdir: Testdir) -> None:
         """Show proper error  when 'request' is used as a parameter name in parametrize (#6183)"""
@@ -136,12 +141,15 @@ class TestMetafunc:
         class DummyFixtureDef:
             scope = attr.ib()
 
-        fixtures_defs = dict(
-            session_fix=[DummyFixtureDef("session")],
-            package_fix=[DummyFixtureDef("package")],
-            module_fix=[DummyFixtureDef("module")],
-            class_fix=[DummyFixtureDef("class")],
-            func_fix=[DummyFixtureDef("function")],
+        fixtures_defs = cast(
+            Dict[str, Sequence[fixtures.FixtureDef]],
+            dict(
+                session_fix=[DummyFixtureDef("session")],
+                package_fix=[DummyFixtureDef("package")],
+                module_fix=[DummyFixtureDef("module")],
+                class_fix=[DummyFixtureDef("class")],
+                func_fix=[DummyFixtureDef("function")],
+            ),
         )
 
         # use arguments to determine narrow scope; the cause of the bug is that it would look on all
@@ -270,7 +278,7 @@ class TestMetafunc:
         deadline=400.0
     )  # very close to std deadline and CI boxes are not reliable in CPU power
     def test_idval_hypothesis(self, value) -> None:
-        escaped = _idval(value, "a", 6, None, item=None, config=None)
+        escaped = _idval(value, "a", 6, None, nodeid=None, config=None)
         assert isinstance(escaped, str)
         escaped.encode("ascii")
 
@@ -292,7 +300,7 @@ class TestMetafunc:
             ),
         ]
         for val, expected in values:
-            assert _idval(val, "a", 6, None, item=None, config=None) == expected
+            assert _idval(val, "a", 6, None, nodeid=None, config=None) == expected
 
     def test_unicode_idval_with_config(self) -> None:
         """unittest for expected behavior to obtain ids with
@@ -321,7 +329,7 @@ class TestMetafunc:
             ("ação", MockConfig({option: False}), "a\\xe7\\xe3o"),
         ]  # type: List[Tuple[str, Any, str]]
         for val, config, expected in values:
-            actual = _idval(val, "a", 6, None, item=None, config=config)
+            actual = _idval(val, "a", 6, None, nodeid=None, config=config)
             assert actual == expected
 
     def test_bytes_idval(self) -> None:
@@ -338,7 +346,7 @@ class TestMetafunc:
             ("αρά".encode(), "\\xce\\xb1\\xcf\\x81\\xce\\xac"),
         ]
         for val, expected in values:
-            assert _idval(val, "a", 6, idfn=None, item=None, config=None) == expected
+            assert _idval(val, "a", 6, idfn=None, nodeid=None, config=None) == expected
 
     def test_class_or_function_idval(self) -> None:
         """unittest for the expected behavior to obtain ids for parametrized
@@ -353,12 +361,10 @@ class TestMetafunc:
 
         values = [(TestClass, "TestClass"), (test_function, "test_function")]
         for val, expected in values:
-            assert _idval(val, "a", 6, None, item=None, config=None) == expected
+            assert _idval(val, "a", 6, None, nodeid=None, config=None) == expected
 
     def test_idmaker_autoname(self) -> None:
         """#250"""
-        from _pytest.python import idmaker
-
         result = idmaker(
             ("a", "b"), [pytest.param("string", 1.0), pytest.param("st-ring", 2.0)]
         )
@@ -373,14 +379,10 @@ class TestMetafunc:
         assert result == ["a0-\\xc3\\xb4"]
 
     def test_idmaker_with_bytes_regex(self) -> None:
-        from _pytest.python import idmaker
-
         result = idmaker(("a"), [pytest.param(re.compile(b"foo"), 1.0)])
         assert result == ["foo"]
 
     def test_idmaker_native_strings(self) -> None:
-        from _pytest.python import idmaker
-
         result = idmaker(
             ("a", "b"),
             [
@@ -414,8 +416,6 @@ class TestMetafunc:
         ]
 
     def test_idmaker_non_printable_characters(self) -> None:
-        from _pytest.python import idmaker
-
         result = idmaker(
             ("s", "n"),
             [
@@ -430,8 +430,6 @@ class TestMetafunc:
         assert result == ["\\x00-1", "\\x05-2", "\\x00-3", "\\x05-4", "\\t-5", "\\t-6"]
 
     def test_idmaker_manual_ids_must_be_printable(self) -> None:
-        from _pytest.python import idmaker
-
         result = idmaker(
             ("s",),
             [
@@ -442,8 +440,6 @@ class TestMetafunc:
         assert result == ["hello \\x00", "hello \\x05"]
 
     def test_idmaker_enum(self) -> None:
-        from _pytest.python import idmaker
-
         enum = pytest.importorskip("enum")
         e = enum.Enum("Foo", "one, two")
         result = idmaker(("a", "b"), [pytest.param(e.one, e.two)])
@@ -451,7 +447,6 @@ class TestMetafunc:
 
     def test_idmaker_idfn(self) -> None:
         """#351"""
-        from _pytest.python import idmaker
 
         def ids(val: object) -> Optional[str]:
             if isinstance(val, Exception):
@@ -471,7 +466,6 @@ class TestMetafunc:
 
     def test_idmaker_idfn_unique_names(self) -> None:
         """#351"""
-        from _pytest.python import idmaker
 
         def ids(val: object) -> str:
             return "a"
@@ -492,7 +486,6 @@ class TestMetafunc:
         disable_test_id_escaping_and_forfeit_all_rights_to_community_support
         option. (#5294)
         """
-        from _pytest.python import idmaker
 
         class MockConfig:
             def __init__(self, config):
@@ -525,7 +518,6 @@ class TestMetafunc:
         disable_test_id_escaping_and_forfeit_all_rights_to_community_support
         option. (#5294)
         """
-        from _pytest.python import idmaker
 
         class MockConfig:
             def __init__(self, config):
@@ -607,16 +599,12 @@ class TestMetafunc:
         )
 
     def test_idmaker_with_ids(self) -> None:
-        from _pytest.python import idmaker
-
         result = idmaker(
             ("a", "b"), [pytest.param(1, 2), pytest.param(3, 4)], ids=["a", None]
         )
         assert result == ["a", "3-4"]
 
     def test_idmaker_with_paramset_id(self) -> None:
-        from _pytest.python import idmaker
-
         result = idmaker(
             ("a", "b"),
             [pytest.param(1, 2, id="me"), pytest.param(3, 4, id="you")],
@@ -625,8 +613,6 @@ class TestMetafunc:
         assert result == ["me", "you"]
 
     def test_idmaker_with_ids_unique_names(self) -> None:
-        from _pytest.python import idmaker
-
         result = idmaker(
             ("a"), map(pytest.param, [1, 2, 3, 4, 5]), ids=["a", "a", "b", "c", "b"]
         )
@@ -689,7 +675,7 @@ class TestMetafunc:
             fail.Exception,
             match="In func: expected Sequence or boolean for indirect, got dict",
         ):
-            metafunc.parametrize("x, y", [("a", "b")], indirect={})  # type: ignore[arg-type] # noqa: F821
+            metafunc.parametrize("x, y", [("a", "b")], indirect={})  # type: ignore[arg-type]
 
     def test_parametrize_indirect_list_functional(self, testdir: Testdir) -> None:
         """
